/*	$Id: dpotf2.c 2007/08/24 13:56:34 mallet $ */

/* Copyright (c) 1992-2006 The University of Tennessee.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer listed
 *   in this license in the documentation and/or other materials
 *   provided with the distribution.
 *
 * - Neither the name of the copyright holders nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* dpotf2.f -- translated by f2c (version 20000817)
 *
 *  -- LAPACK routine (version 3.1) --
 *     Univ. of Tennessee, Univ. of California Berkeley and NAG Ltd..
 *     November 2006
 */

/* f2c translation reworked (no algorithm changes) by Anthony Mallet
 * August 2007 */
#include <math.h>

typedef int	integer;
typedef double	doublereal;


/* ------------------------------------------------------------------- */

int
pom_dpotf2(integer n, doublereal *a, integer lda, integer *info)
{
    /* Local variables */
    extern doublereal pom_ddot(integer n, doublereal *dx, doublereal *dy);
    extern int pom_dscal(integer n, doublereal da, doublereal *dx, integer incx);
    extern int pom_dgemv(integer m, integer n, doublereal *a, integer lda,
			 doublereal *x, doublereal *y, integer incy);

    integer j;
    doublereal ajj;


/*  -- LAPACK routine (version 3.1) -- */
/*     Univ. of Tennessee, Univ. of California Berkeley and NAG Ltd.. */
/*     November 2006 */

/*  Purpose */
/*  ======= */

/*  DPOTF2 computes the Cholesky factorization of a real symmetric */
/*  positive definite matrix A. */

/*  The factorization has the form */
/*     A = U' * U */
/*  where U is an upper triangular matrix. */

/*  This is the unblocked version of the algorithm, calling Level 2 BLAS. */

/*  Arguments */
/*  ========= */

/*  N       (input) INTEGER */
/*          The order of the matrix A.  N >= 0. */

/*  A       (input/output) DOUBLE PRECISION array, dimension (LDA,N) */
/*          On entry, the symmetric matrix A.  the leading */
/*          n by n upper triangular part of A contains the upper */
/*          triangular part of the matrix A, and the strictly lower */
/*          triangular part of A is not referenced. */

/*          On exit, if INFO = 0, the factor U from the Cholesky */
/*          factorization A = U'*U. */

/*  LDA     (input) INTEGER */
/*          The leading dimension of the array A.  LDA >= max(1,N). */

/*  INFO    (output) INTEGER */
/*          = 0: successful exit */
/*          > 0: if INFO = k, the leading minor of order k is not */
/*               positive definite, and the factorization could not be */
/*               completed. */

/*  ===================================================================== */

    /* Parameter adjustments */
    a -= 1 + lda;

    /* Compute the Cholesky factorization A = U'*U. */

    for (j = 1; j <= n; ++j) {

      /* Compute U(J,J) and test for non-positive-definiteness. */
      ajj = a[j + j * lda] -
	pom_ddot(j-1, &a[j * lda + 1], &a[j * lda + 1]);
      if (ajj <= 0.) {
	a[j + j * lda] = ajj;
	goto fail;
      }
      ajj = sqrt(ajj);
      a[j + j * lda] = ajj;

      /* Compute elements J+1:N of row J. */
      if (j < n) {
	pom_dgemv(j-1, n-j, &a[(j + 1) * lda + 1], lda,
		  &a[j * lda + 1], &a[j + (j + 1) * lda], lda);
	pom_dscal(n - j, 1.0/ajj, &a[j + (j + 1) * lda], lda);
      }
    }
    return 0;

fail:
    *info = j;
    return 1;
} /* dpotf2_ */


/* ------------------------------------------------------------------- */

/* dgemv.f -- translated by f2c (version 20000817). */

/* f2c translation reworked and simplified by Anthony Mallet
 * August 2007 */

int
pom_dgemv(integer m, integer n, doublereal *a, integer lda,
	  doublereal *x, doublereal *y, integer incy)
{
    /* Local variables */
    doublereal temp;
    integer i, j;
    integer jy;

/*  Purpose */
/*  ======= */

/*  DGEMV  performs y := -A'*x + y, */

/*  where x and y are vectors and A is an m by n matrix. */

/*  Arguments */
/*  ========== */

/*  M      - INTEGER. */
/*           On entry, M specifies the number of rows of the matrix A. */
/*           M must be at least zero. */
/*           Unchanged on exit. */

/*  N      - INTEGER. */
/*           On entry, N specifies the number of columns of the matrix A. */
/*           N must be at least zero. */
/*           Unchanged on exit. */

/*  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ). */
/*           Before entry, the leading m by n part of the array A must */
/*           contain the matrix of coefficients. */
/*           Unchanged on exit. */

/*  LDA    - INTEGER. */
/*           On entry, LDA specifies the first dimension of A as declared */
/*           in the calling (sub) program. LDA must be at least */
/*           max( 1, m ). */
/*           Unchanged on exit. */

/*  X      - DOUBLE PRECISION array of DIMENSION at least m. */
/*           Before entry, the incremented array X must contain the */
/*           vector x. */
/*           Unchanged on exit. */

/*  Y      - DOUBLE PRECISION array of DIMENSION at least */
/*           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n' */
/*           and at least */
/*           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise. */
/*           Before entry, the incremented array Y */
/*           must contain the vector y. On exit, Y is overwritten by the */
/*           updated vector y. */

/*  INCY   - INTEGER. */
/*           On entry, INCY specifies the increment for the elements of */
/*           Y. INCY must not be zero. */
/*           Unchanged on exit. */


/*  Level 2 Blas routine. */

/*  -- Written on 22-October-1986. */
/*     Jack Dongarra, Argonne National Lab. */
/*     Jeremy Du Croz, Nag Central Office. */
/*     Sven Hammarling, Nag Central Office. */
/*     Richard Hanson, Sandia National Labs. */


/*     Test the input parameters. */

    /* Parameter adjustments */
    a -= 1 + lda;
    --x;
    --y;

/*     Start the operations. In this version the elements of A are */
/*     accessed sequentially with one pass through A. */

/*        Form  y := -A'*x + y. */

    jy = 1;
    for (j = 1; j <= n; ++j) {
      temp = 0.;
      for (i = 1; i <= m; ++i) {
	temp += a[i + j * lda] * x[i];
      }
      y[jy] -= temp;
      jy += incy;
    }

    return 0;
} /* dgemv_ */


/* ------------------------------------------------------------------- */

/* ddot.f -- translated by f2c (version 20000817). */

/* f2c translation reworked (no algorithm changes) by Anthony Mallet
 * August 2007 */

doublereal
pom_ddot(integer n, doublereal *dx, doublereal *dy)
{
    /* System generated locals */
    doublereal ret_val;

    /* Local variables */
    integer i, m;
    doublereal dtemp;

/*  Purpose */
/*  ======= */

/*     forms the dot product of two vectors. */
/*     uses unrolled loops for increments equal to one. */
/*     jack dongarra, linpack, 3/11/78. */
/*     modified 12/3/93, array(1) declarations changed to array(*) */

    /* Parameter adjustments */
    --dy;
    --dx;

    /* Function Body */
    ret_val = 0.;
    dtemp = 0.;

/*        clean-up loop */
    m = n % 5;
    if (m == 0) {
	goto L40;
    }
    for (i = 1; i <= m; ++i) {
	dtemp += dx[i] * dy[i];
/* L30: */
    }
    if (n < 5) {
	goto L60;
    }
L40:
    for (i = m+1; i <= n; i += 5) {
	dtemp = dtemp + dx[i] * dy[i] + dx[i + 1] * dy[i + 1] +
	  dx[i + 2] * dy[i + 2] + dx[i + 3] * dy[i + 3] + dx[i + 4] * dy[i + 4];
/* L50: */
    }
L60:
    ret_val = dtemp;
    return ret_val;
} /* ddot_ */


/* ------------------------------------------------------------------- */

/* dscal.f -- translated by f2c (version 20000817). */

/* f2c translation reworked (no algorithm changes) by Anthony Mallet
 * August 2007 */

int
pom_dscal(integer n, doublereal da, doublereal *dx, integer incx)
{
    /* Local variables */
    integer i, nincx;

/*  Purpose */
/*  ======= */
/* * */
/*     scales a vector by a constant. */
/*     jack dongarra, linpack, 3/11/78. */
/*     modified 3/93 to return if incx .le. 0. */
/*     modified 12/3/93, array(1) declarations changed to array(*) */

    /* Parameter adjustments */
    --dx;

    /* Function Body */
    nincx = n * incx;
    for (i = 1; incx < 0 ? i >= nincx : i <= nincx; i += incx) {
	dx[i] = da * dx[i];
    }
    return 0;
} /* dscal_ */
