/*	$Id: log.c 2007/08/22 18:56:03 mallet $ */

/*
 * Copyright (c) 2003-2007 LAAS/CNRS                  --  Fri Sep  1 2000
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
static const char id[] = "$Id: log.c 2007/08/22 18:56:03 mallet $";

#ifdef VXWORKS
#include <vxWorks.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "t3d/t3d.h"

#include "libpom/lib.h"
#include "pomInt.h"

int
pomWrite(PomMESet *set, int date)
{
   int i, j, c , k;
   T3D *t3 = t3dNew(T3D_U_MATRIX, T3D_ALLOW_CONVERSION);

   gzprintf(set->log, "date %d\n", set->currentTime-date);

   gzprintf(set->log, "*********** fused **************\nbaseToOrigin = ");
   t3dConvertTo(T3D_EULER,&set->me[0].baseToOrigin);
   for(j=0;j<6;j++)
     gzprintf(set->log, "%f ",
	      set->me[0].baseToOrigin.euler.euler[j]);
   gzprintf(set->log, "\n");

   gzprintf(set->log, "originToZero = ");
   t3dConvertTo(T3D_EULER,&set->originToZero);
   for(j=0;j<6;j++)
     gzprintf(set->log, "%f ", set->originToZero.euler.euler[j]);
   gzprintf(set->log, "\n");

   gzprintf(set->log, "mainToOrigin = ");
   t3dConvertTo(T3D_EULER,&set->me[0].p[date]->pos.pos.abs.mainToOrigin);
   for(j=0;j<6;j++)
     gzprintf(set->log, "%f ",
	      set->me[0].p[date]->
	      pos.pos.abs.mainToOrigin.euler.euler[j]);
   gzprintf(set->log, "\n");
   for(c=0,j=0;j<6;j++) {
       gzprintf(set->log,"\t |");
       for(k=0,c+=j;k<6;k++)
	 if (k>j) gzprintf(set->log,"         ");
	 else gzprintf(set->log,"%8.4f ", set->me[0].p[date]->pos.pos.abs.mainToOrigin.euler.v[c+k]);
       gzprintf(set->log,"|\n");
     }
   gzprintf(set->log, "\n");
   gzprintf(set->log, "mainToZero = ");
   t3dCompIn(t3,&set->me[0].p[date]->pos.pos.abs.mainToOrigin,&set->originToZero);
   t3dConvertTo(T3D_EULER,t3);
   for(j=0;j<6;j++)
     gzprintf(set->log, "%f ", t3->euler.euler[j]);
   gzprintf(set->log, "\n");
   gzprintf(set->log, "npos = %d\n", set->nmes-1);

   for(i=1;i<set->nmes;i++) {
     if (set->me[i].p[date] == NULL) continue;
     gzprintf(set->log, "************  %s  *********\n", set->me[i].name);
     gzprintf(set->log, "fuse = %d\nconfidence = %f\n",
	      set->me[i].fuse,
	      set->me[i].p[date]->pos.confidence);

     gzprintf(set->log, "mainToOrigin = ");
     t3dConvertTo(T3D_EULER,&set->me[i].p[date]->pos.pos.abs.mainToOrigin);
     for(j=0;j<6;j++)
       gzprintf(set->log, "%f ",
		set->me[i].p[date]->pos.pos.abs.mainToOrigin.euler.euler[j]);
     gzprintf(set->log, "\n");
     gzprintf(set->log,"Covariance sur mainToOrigin = \n");

     for(c=0,j=0;j<6;j++) {
       gzprintf(set->log,"\t |");
       for(k=0,c+=j;k<6;k++)
	 if (k>j) gzprintf(set->log,"         ");
	 else gzprintf(set->log,"%8.4f ", set->me[i].p[date]->pos.pos.abs.mainToOrigin.euler.v[c+k]);
       gzprintf(set->log,"|\n");
     }
   }
   t3dDelete(t3);
   return 0;
}
