/*
 * Copyright (C) 2000 LAAS/CNRS
 *
 * Anthony Mallet - Thu Jun 29 2000
 *
 * $Log$
 * Revision 1.2  2005/08/31 15:04:00  sara
 * include zlib.h
 *
 * Revision 1.1.1.1  2005/06/16 12:36:51  matthieu
 * import autotoolized calife subset needed by pom
 *
 * Revision 1.2.2.4  2004/03/25 14:08:11  mcottret
 * Change/add some structures for the new-look pom
 *
 * Revision 1.2.2.3  2003/07/31 08:05:19  jbhayet
 * Des vitesses, des prints, une nouvelle structure
 *
 * Revision 1.2.2.2  2003/06/03 11:48:31  jbhayet
 * Petits reglages
 *
 * Revision 1.2.2.1  2003/05/28 08:03:36  jbhayet
 * Gestion des incertitudes
 *
 * Revision 1.2  2000/09/04 19:17:40  mallet
 * version qui fusionne par ecrasement en accordant la priorite aux estimateurs dont la confiance est la plus grande
 *
 * Revision 1.1  2000/08/17 14:26:50  mallet
 * devel
 *
 * Revision 1.1  2000/07/24 16:44:09  mallet
 * Bibliothque pour l'estimation de position par fusion de plusieurs estime
 * provenant de plusieurs estimateurs diffrents.
 * Coquille un peu vide pour l'instant.
 *
 */
#ifndef _POMINT_H
#define _POMINT_H

#include "libpom/struct.h"
#include "zlib.h"  /* gzprintf, gzFile, ... */

/* Chained list of LibPomMEPos ------------------------------------------ */

typedef struct PomMEList {
   LibPomMEPos pos;	/* position */
   struct PomMEList *next;
} PomMEList;


/* History of positions ---------------------------------------------- */

typedef struct PomMEHistory {
  int fuse;		/* true if postion is to be fused */
  int isOffset;	/* true if offset (below) is set */
  char *name;		/* name of position estimator */
  PomMEList **p;	/* history of positions */

  T3D offset;		/* first position of estimator */
  T3D baseToOrigin;	/* offset (position drift after corrections) */
  PomMEAbsPos lastabs;	/* The last pose we used to compute basetoorigin */
} PomMEHistory;

typedef struct PomMESet {
   int currentTime;	/* internal date */
   int fusionIndex;	/* last fusion */

   int length;		/* length of arrays */
   int nmes;		/* number of position estimators */

  PomFusionMethod method;/* The fusion method */
  int packets;		/* size of arrays */

  PomMEHistory *me;	/* array of position estimators */
  int refMe;		/* reference ME (usually the fastest) */

  T3D originToZero;	/* offset (pomSetZero) */

  gzFile *log;		/* log file */
} PomMESet;


/* --- Private functions --------------------------------------------- */

int	pomBestME(PomMESet *set, int date);
int	pomDateIndex(PomMESet *set, int date);
int	pomIntegrateBaseToOrigin(T3D *m2o, PomMESet *set, int me, int index);
int	pomKalmanUpdateB2O(PomMESet *set, int index, int *nfused, T3D *m2o);

T3D *	pomRefDisplacement(PomMESet *set, int date1, int date2);

int	pomUpdateB2O(PomMESet *set, int me, int index, T3D *m2o);
int	pomSingleUpdateB2O(T3D *m2o, PomMESet *set, int me, int index);
int	pomWrite(PomMESet *set, int date);

PomMEClass	pomMEClass(PomMESet *set, int me);
int		pomMELastPos(PomMESet *set, int me, int index);
int		pomMESetResizeL(PomMESet *set, int oldest);
int		pomMESetResizeN(PomMESet *set, int nmes);

void	pomDerivativeOfEuler(double *euler, double *dRa, double *dRb,
		double *dRg);
void	pomMatrixOfEuler(double *euler, double *mat);

int	check_cov_matrix(double *cov, gzFile *log);
int	choldc_sym_without_nrerror(double **a, double **b, int n);
int	dms_inv_cov_without_nrerror(double *m1, double *m2);
int	dms_vp_without_nrerror(double *m, double *valp, double *vectp, int l, int c);

int	pom_dm_fill(double *mat, int l, int c, ...);
int	pom_dms_fill(double *mat, int l, int c, ...);
int	pom_dm_trans_3x3(double *m, double *r);
int	pom_dm_trans_6x6(double *m, double *r);
int	pom_dms_mulscal_6x6(double *m, double scal);
int	pom_dm_mult(const double *m1, const double *m2, double *m,
		int l1, int c1, int l2, int c2);
int	pom_dmsf_mult(double *m1, double *m2, double *m, int l1,
		int c1, int l2, int c2);
int	pom_dms_plus_6x6(double *m1, double *m2, double *m);
int	pom_dm_plustr_6x6(double *m1, double *m);
int	pom_dmsf_multtr_6x6(double *m1, double *m2, double *m);
int	pom_dmfsf_jmult_6x6(double *m1, double *m2, double *m);
int	pom_dms_inv_6x6(double *m1, double *m2);
int	pom_dpotf2(int n, double *a, int lda, int *info);

#endif /* _POMINT_H */
