/*	$Id: utils.c 2007/08/24 16:42:07 mallet $ */

/*
 * Copyright (c) 2003-2007 LAAS/CNRS                  --  Fri Sep  1 2000
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
static const char id[] = "$Id: utils.c 2007/08/24 16:42:07 mallet $";

#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "libpom/lib.h"
#include "pomInt.h"


#define VAR_CALC_MIN_VALUE 0.0005

/* Only for 6x6 covariance matrix */
int
check_cov_matrix(double *cov, gzFile *log)
{
  int j,k,c, r, info;
  double m[9];

  /* extract 3x3 matrix for yaw, x, y */
  m[0] =	cov[0];
  m[1] = m[3] = cov[6];
  m[2] = m[6] = cov[10];
  m[4] =	cov[9];
  m[5] = m[7] = cov[13];
  m[8] =	cov[14];

  /* try cholesky factorization */
  r = pom_dpotf2(3, m, 3, &info);
  if (!r) return 0;

  fprintf(stderr, "covariance matrix not invertible!\n");
  if (log!=NULL) {
    gzprintf(log,"Check this matrix !\n");
    for(c=0,j=0;j<6;j++) {
      gzprintf(log,"\t |");
      for(k=0,c+=j;k<6;k++)
	if (k>j) gzprintf(log,"         ");
	else gzprintf(log,"%8.4f ", cov[c+k]);
      gzprintf(log,"|\n");
    }
  }
  return 1;
}
